
    /*!
    * scroll-watcher - v2.1.0
    * Built: Mon Feb 22 2021 11:18:33 GMT-0300 (Brasilia Standard Time)
    */
  
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.ScrollWatcher = factory());
}(this, (function () { 'use strict';

  var EVENT_TYPE = {
    PAGELOAD: 'page:load',
    SCROLLING: 'scrolling',
    ENTER: 'enter',
    FULL_ENTER: 'enter:full',
    EXIT_PARTIAL: 'exit:partial',
    EXIT: 'exit',
  };

  var DEFAULT_OFFSET = {
    top: 0,
    bottom: 0,
  };

  function assert(condition, message) {
    if ( message === void 0 ) message = 'Assertion failed';

    if (!condition) {
      if (typeof Error !== 'undefined') { throw new Error(message); }

      throw message; // Fallback
    }
  }

  function resetProperties(item) {
    item.isInViewport = false;
    item.wasInViewport = false;
    item.isAboveViewport = false;
    item.wasAboveViewport = false;
    item.isBelowViewport = false;
    item.wasBelowViewport = false;
    item.isPartialOut = false;
    item.wasPartialOut = false;
    item.isFullyOut = false;
    item.wasFullyOut = false;
    item.isFullyInViewport = false;
    item.wasFullyInViewport = false;
  }

  function resetPartialProperties(item) {
    item.isInViewport = false;
    item.isAboveViewport = false;
    item.isBelowViewport = false;
    item.isPartialOut = false;
    item.isFullyOut = false;
    item.isFullyInViewport = false;
  }

  function getProperties(item) {
    return {
      isInViewport: item.isInViewport,
      isFullyInViewport: item.isFullyInViewport,
      isAboveViewport: item.isAboveViewport,
      isBelowViewport: item.isBelowViewport,
      isPartialOut: item.isPartialOut,
      isFullyOut: item.isFullyOut,
    };
  }

  function fireEvents(item, data) {
    if (item.isInViewport && !item.wasInViewport) {
      item.wasInViewport = true;
      item.wasFullyOut = false;
      item.emitter.emit(EVENT_TYPE.ENTER, data);
    }

    if (item.isPartialOut && !item.wasPartialOut) {
      item.wasPartialOut = true;
      item.wasFullyInViewport = false;
      item.emitter.emit(EVENT_TYPE.EXIT_PARTIAL, data);
    }

    if (item.isFullyOut && !item.wasFullyOut) {
      item.wasFullyOut = true;
      item.wasInViewport = false;
      item.wasFullyInViewport = false;
      item.emitter.emit(EVENT_TYPE.EXIT, data);
    }

    if (item.isFullyInViewport && !item.wasFullyInViewport) {
      item.wasFullyInViewport = true;
      item.wasPartialOut = false;
      item.wasFullyOut = false;
      item.emitter.emit(EVENT_TYPE.FULL_ENTER, data);
    }
  }

  /**
   * @param {Element|Array<Element>} element DOM node or array of nodes.
   * @param {String|Array<String>} classname Class or array of classes.
   * For example: 'class1 class2' or ['class1', 'class2']
   */

  function getScroll() {
    return [
      window.pageXOffset ||
        (document.documentElement && document.documentElement.scrollLeft) ||
        document.body.scrollLeft,
      window.pageYOffset ||
        (document.documentElement && document.documentElement.scrollTop) ||
        document.body.scrollTop ];
  }

  function getViewportSize() {
    return {
      w: window.innerWidth || document.documentElement.clientWidth,
      h: window.innerHeight || document.documentElement.clientHeight,
    };
  }

  function getDocumentHeight() {
    return Math.max(
      document.body.scrollHeight,
      document.documentElement.scrollHeight,
      document.body.offsetHeight,
      document.documentElement.offsetHeight,
      document.documentElement.clientHeight
    );
  }

  function getOffset(element) {
    var rect = element.getBoundingClientRect();
    var documentElement = document.documentElement;
    var left = rect.left + window.pageXOffset - documentElement.clientLeft;
    var top = rect.top + window.pageYOffset - documentElement.clientTop;
    var width = element.offsetWidth;
    var height = element.offsetHeight;
    var right = left + width;
    var bottom = top + height;

    return { width: width, height: height, top: top, bottom: bottom, right: right, left: left };
  }

  function isElement(object) {
    // DOM, Level2
    if ('HTMLElement' in window) {
      return !!object && object instanceof HTMLElement;
    }

    // Older browsers
    return !!object && typeof object === 'object' && object.nodeType === 1 && !!object.nodeName;
  }

  /**
   * Based on https://github.com/developit/mitt
   */

  function Mitt (all) {
    if ( all === void 0 ) all = new Map();

    return {
      all: all,

      /**
       * Register an event handler for the given type.
       * @param {string|symbol} type Type of event to listen for, or `"*"` for all events
       * @param {Function} handler Function to call in response to given event
       */
      on: function on(type, handler) {
        var handlers = all.get(type);
        var added = handlers && handlers.push(handler);

        if (!added) {
          all.set(type, [handler]);
        }
      },

      /**
       * Remove an event handler for the given type.
       * @param {string|symbol} type Type of event to unregister `handler` from, or `"*"`
       * @param {Function} handler Handler function to remove
       */
      off: function off(type, handler) {
        var handlers = all.get(type);

        if (handlers) {
          handlers.splice(handlers.indexOf(handler), 1);
        }
      },

      /**
       * Invoke all handlers for the given type.
       * If present, `"*"` handlers are invoked after type-matched handlers.
       *
       * Note: Manually firing "*" handlers is not supported.
       *
       * @param {string|symbol} type The event type to invoke
       * @param {Any} [evt] Any value (object is recommended and powerful), passed to each handler
       */
      emit: function emit(type, evt) {
        (all.get(type) || []).slice().map(function (handler) { return handler(evt); });
        (all.get('*') || []).slice().map(function (handler) { return handler(type, evt); });
      },
    };
  }

  function base () {
    var watcher = {
      counter: 0,
      requestAnimationId: null,
      lastXY: [],
      watching: {},
      viewport: getViewportSize(),
      props: null,
    };

    var emitter = new Mitt();

    function initialize() {
      handleAnimationFrame();

      var onReadyState = function () {
        if (typeof document !== 'undefined') {
          switch (document.readyState) {
            case 'loading':
            case 'interactive':
              break;
            case 'complete':
              emitter.emit(EVENT_TYPE.PAGELOAD, {
                scrollX: watcher.lastXY[0],
                scrollY: watcher.lastXY[1],
              });
              document.removeEventListener('readystatechange', onReadyState);

              break;
          }
        }
      };

      document.addEventListener('readystatechange', onReadyState, false);
      window.addEventListener(
        'resize',
        function () {
          watcher.viewport = getViewportSize();
        },
        false
      );
    }

    function watch(element, initOffset) {
      var node = isElement(element) ? element : document.querySelector(element);

      assert(isElement(node), "Couldn't find target in DOM");
      assert(
        typeof initOffset === 'number' ||
          typeof initOffset === 'object' ||
          typeof initOffset === 'undefined',
        '@param `initOffset` should be number or Object or undefined!'
      );

      var optionsOffset =
        typeof initOffset === 'number'
          ? { top: initOffset, bottom: initOffset }
          : Object.assign(DEFAULT_OFFSET, initOffset);

      var watchingEmitter = new Mitt();
      var watching = {
        node: node,
        offset: optionsOffset,
        dimensions: getOffset(node),
        emitter: watchingEmitter,
      };

      watcher.counter += 1;
      watcher.watching[watcher.counter] = watching;
      resetProperties(watching);

      return {
        target: node,
        props: getProperties(watching),

        update: function update() {
          var data = getScrollData();

          data.target = watching.node;
          window.cancelAnimationFrame(watcher.requestAnimationId);
          resetPartialProperties(watching);

          watching.dimensions = getOffset(node);
          recalculate(watching);

          watcher.props = getProperties(watching);
          fireEvents(watching, data);
          handleAnimationFrame();

          return this;
        },

        once: function once(eventName, callback) {
          var wrappedHandler = function (evt) {
            callback(evt);
            watchingEmitter.off(eventName, wrappedHandler);
          };

          watchingEmitter.on(eventName, wrappedHandler);

          return this;
        },

        on: function on(eventName, callback) {
          watchingEmitter.on(eventName, callback);

          return this;
        },

        off: function off(eventName, callback) {
          watchingEmitter.off(eventName, callback);

          return this;
        },
      };
    }

    function getScrollData() {
      var xy = getScroll();
      var scrollingDown = xy[1] > watcher.lastXY[1];

      watcher.lastXY = xy;

      return {
        scrollX: xy[0],
        scrollY: xy[1],
        scrollingDown: scrollingDown,
        scrollingUp: !scrollingDown,
      };
    }

    function recalculate(item) {
      var el = {
        top: item.dimensions.top + item.offset.top,
        bottom: item.dimensions.top + item.offset.bottom + item.dimensions.height,
      };

      var vp = {
        top: watcher.lastXY[1],
        bottom: watcher.lastXY[1] + watcher.viewport.h,
      };

      item.isAboveViewport = el.top < vp.top;
      item.isBelowViewport = el.bottom > vp.bottom;
      item.isInViewport = el.top <= vp.bottom && el.bottom > vp.top;
      item.isFullyInViewport =
        (el.top >= vp.top && el.bottom <= vp.bottom) ||
        (item.isAboveViewport && item.isBelowViewport);
      item.isFullyOut = !item.isInViewport && item.wasInViewport;
      item.isPartialOut = item.wasFullyInViewport && !item.isFullyInViewport && !item.isFullyOut;
    }

    function handleAnimationFrame() {
      var requestId;

      var tick = function () {
        var changed = watcher.lastXY.join() !== getScroll().join();

        if (changed) {
          var evtData = getScrollData();

          emitter.emit(EVENT_TYPE.SCROLLING, evtData);

          Object.keys(watcher.watching).forEach(function (key) {
            evtData.target = watcher.watching[key].node;
            recalculate(watcher.watching[key]);
            fireEvents(watcher.watching[key], evtData);
          });
        }

        requestId = window.requestAnimationFrame(tick);
      };

      requestId = window.requestAnimationFrame(tick);

      return requestId;
    }

    /**
     * @param {Number|undefined} offset How far to offset.
     * @return {Boolean} Whether window is scrolled to bottom
     */
    function windowAtBottom(offset) {
      if ( offset === void 0 ) offset = 0;

      var scrolled = watcher.lastXY[1];
      var viewHeight = watcher.viewport.h;

      // eslint-disable-next-line no-param-reassign
      offset = Number.parseInt(offset, 10);

      return scrolled + viewHeight >= getDocumentHeight() - offset;
    }

    /**
     * @param {Number|undefined} offset How far to offset.
     * @return {Boolean} Whether window is scrolled to top
     */
    function windowAtTop(offset) {
      if ( offset === void 0 ) offset = 0;

      // eslint-disable-next-line no-param-reassign
      offset = Number.parseInt(offset, 10);

      return watcher.lastXY[1] <= offset;
    }

    return { initialize: initialize, watch: watch, emitter: emitter, windowAtBottom: windowAtBottom, windowAtTop: windowAtTop };
  }

  function entry () {
    var ref = base();
    var initialize = ref.initialize;
    var watch = ref.watch;
    var windowAtBottom = ref.windowAtBottom;
    var windowAtTop = ref.windowAtTop;
    var emitter = ref.emitter;

    initialize();

    return {
      watch: watch,
      on: emitter.on,
      off: emitter.off,
      emit: emitter.emit,
      windowAtBottom: windowAtBottom,
      windowAtTop: windowAtTop,
    };
  }

  return entry;

})));
